#!/usr/bin/env jruby

######################################
#                                    #
#  RedTeam Pentesting GmbH           #
#  kontakt@redteam-pentesting.de     #
#  http://www.redteam-pentesting.de  #
#                                    #
######################################

require 'java'
require '../jboss_jars/jbossall-client.jar'

require 'optparse'
require 'ostruct'

import 'java.util.Hashtable'
import 'javax.naming.InitialContext'
import 'javax.management.MBeanServerConnection'
import 'javax.management.ObjectName'

class HttpInvoker

  def initialize
    @options = parseopts(ARGV)
    ctx = get_ctx(@options.jndi)
    server = ctx.lookup(@options.adaptor);
    obj = ObjectName.new(@options.mbean)
    # TODO
    #if @options.attribute
    #  puts get_attribute(obj, @options.attribute)
    #end
    if @options.invoke_method or @options.test
      puts invoke(server, obj, @options.invoke_method, @options.invoke_params, @options.invoke_sig)
    end
  end

  def parseopts(args)
    options = OpenStruct.new({
      :jndi => "http://localhost:8080/invoker/JNDIFactory",
      :adaptor => "jmx/invoker/HttpAdaptor",
      :test => false,
      :invoke_params => [].to_java(:Object),
      :invoke_sig => [].to_java(:String)
    })
    opts = OptionParser.new do |opts|
      opts.banner = "Usage: #{__FILE__} [options] MBean"
      opts.separator ""
      opts.on("-j", "--jndi URL", "The JNDI URL to use (default: #{options.jndi})") do |url|
        options.jndi = url 
      end
      opts.on("-p", "--adaptor URL", "The Adaptor URL to use (default: #{options.adaptor})") do |url|
        options.adaptor = url 
      end
      # TODO
      #opts.on("-a", "--get-attr ATTR", "Read an attribute of an MBean") do |attribute|
      #  options.attribute = attribute
      #end
      opts.on("-i", "--invoke METHOD", "invoke an MBean method") do |m|
        options.invoke_method = m
      end
      opts.on("-p", "--invoke-params PARAMS", Array, "MBean method params") do |params|
        params.map! {|x| if ["true", "false"].include? x then eval(x) else x end}
        options.invoke_params = params.to_java(:Object)
      end
      opts.on("-s", "--invoke-sigs SIGS", Array, "MBean method signature") do |sigs|
        options.invoke_sig = sigs.to_java(:String)
      end
      opts.on("-t", "--test", "Test the script with the ServerInfo MBean's listThreadDump() method") do
        options.test = true
        options.mbean = "jboss.system:type=ServerInfo"
        options.invoke_method = "listThreadDump"
      end
      opts.on("-h", "--help", "Show this help") do
        puts opts
        exit
      end
    end
    opts.parse! args
    unless options.test
      unless ARGV[0]
        puts "Please give the name of the MBean you want to access"
        exit 1
      end
      options.mbean = ARGV[0]
    end
    options
  end


  def get_ctx(jndi)
    table = Hashtable.new()
    table.put("java.naming.factory.initial","org.jboss.naming.HttpNamingContextFactory")
    table.put("java.naming.provider.url", jndi)
    table.put("java.naming.factory.url.pkgs","org.jboss.naming.client")
    return InitialContext.new(table)
  end


  def invoke(server, t_obj_name, action_name, params, signature)
    server.invoke(t_obj_name, action_name, params, signature)
  end

end

if $0 == __FILE__
  HttpInvoker.new
end
