#!/usr/bin/env jruby

######################################
#                                    #
#  RedTeam Pentesting GmbH           #
#  kontakt@redteam-pentesting.de     #
#  http://www.redteam-pentesting.de  #
#                                    #
######################################

require 'java'
require '../jboss_jars/console-mgr-classes.jar'
require '../jboss_jars/jbossall-client.jar'

require 'optparse'
require 'ostruct'

import 'java.net.URL'
import 'javax.management.ObjectName'
import 'org.jboss.console.remote.Util'
import 'org.jboss.console.remote.RemoteMBeanAttributeInvocation'
import 'org.jboss.console.remote.RemoteMBeanInvocation'


class WebconsoleInvoker

  def parseopts(args)
    options = OpenStruct.new({
      :url => "http://localhost:8080/web-console/Invoker",
      :test => false,
      :invoke_params => [].to_java(:Object),
      :invoke_sig => [].to_java(:String)
    })
    opts = OptionParser.new do |opts|
      opts.banner = "Usage: #{__FILE__} [options] MBean"
      opts.separator ""
      opts.on("-u", "--url URL", "The Invoker URL to use (default: #{options.url})") do |url|
        options.url = url
      end
      opts.on("-a", "--get-attr ATTR", "Read an attribute of an MBean") do |attribute|
        options.attribute = attribute
      end
      opts.on("-i", "--invoke METHOD", "invoke an MBean method") do |m|
        options.invoke_method = m
      end
      opts.on("-p", "--invoke-params PARAMS", Array, "MBean method params") do |params|
        params.map! {|x| if ["true", "false"].include? x then eval(x) else x end}
        options.invoke_params = params.to_java(:Object)
      end
      opts.on("-s", "--invoke-sigs SIGS", Array, "MBean method signature") do |sigs|
        options.invoke_sig = sigs.to_java(:String)
      end
      opts.on("-t", "--test", "Test the script with the ServerInfo MBean's listThreadDump() method") do
        options.test = true
        options.mbean = "jboss.system:type=ServerInfo"
        options.attribute = "OSVersion"
        options.invoke_method = "listThreadDump"
      end
      opts.on("-h", "--help", "Show this help") do
        puts opts
        exit
      end
      opts.separator ""
      opts.separator "Example usage:"
      opts.separator "#{__FILE__} -a OSVersion jboss.system:type=ServerInfo"
      opts.separator "#{__FILE__} -i listThreadDump jboss.system:type=ServerInfo"
      opts.separator "#{__FILE__} -i listMemoryPools -p true -s boolean jboss.system:type=ServerInfo"
      opts.separator ""
      opts.separator "As params, only Strings and booleans are allowed. This is due to the fact that"
      opts.separator "we want to be able to give the data structure on the command line. Numbers may"
      opts.separator "be supported next."
      opts.separator ""
    end
    opts.parse! args
    unless options.test
      unless ARGV[0]
        puts "Please give the name of the MBean you want to access"
        exit 1
      end
      options.mbean = ARGV[0]
    end
    options
  end

  def initialize
    @options = parseopts(ARGV)
    obj = ObjectName.new(@options.mbean)
    if @options.attribute or @options.test
      puts get_attribute(obj, @options.attribute)
    end
    if @options.invoke_method or @options.test
      puts invoke(obj, @options.invoke_method, @options.invoke_params, @options.invoke_sig)
    end
  end

  # Return the string representation of the attribute +attr_name+ of MBean
  # +t_obj_name+
  def get_attribute(t_obj_name, attr_name)
    mi = RemoteMBeanAttributeInvocation.new(t_obj_name, attr_name)
    Util.getAttribute(URL.new(@options.url), mi)
  end

  # Return the string representation of the invocation result of method
  # +action_name+ with params +params+ of the MBean +t_obj_name+
  def invoke(t_obj_name, action_name, params, signature)
    mi = RemoteMBeanInvocation.new(t_obj_name, action_name, params, signature)
    Util.invoke(URL.new(@options.url), mi)
  end

end

if $0 == __FILE__
  WebconsoleInvoker.new
end
